import terrapin.*;
import megamu.mesh.*;
import processing.pdf.*;


int XSIZE=int(3.6*297);
int YSIZE=int(3.6*210);
PFont font16, font32;
PGraphicsPDF pdf;

void setup() {
     noLoop();
     size(XSIZE,YSIZE);
     pdf = (PGraphicsPDF) beginRecord(PDF, "NEWVORONOI.pdf");
    //this was tricky, see JunTestFont or email 20-12-2011
}

class PPoint extends PVector {
      PPoint(float x, float y){
            super(x,y);
      }
      //the following method does not change "this" but makes a new point
      //I would have preferred a static method,  but that was not allowed
      //inpol interpolates "this" and B, to give a middle point, half way 
      PPoint intpol(PPoint B){
             PVector M = mult(PPoint.add(this,B),0.5);
             return new PPoint(M.x,M.y);
      }
      PPoint intpol(PPoint B, float f){
             PVector M = PPoint.add(mult(this,1-f),mult(B,f));
             return new PPoint(M.x,M.y);
      }
}

void setPosition(Terrapin t, PPoint P){
     t.x = int(P.x);//GRRR int
     t.y = int(P.y);
}
PPoint getPosition(Terrapin t){
     return new PPoint(t.x,t.y);
}
void setRotation(Terrapin t, int theta){
     t.setRotation(theta);
}
int   getRotation(Terrapin t){
      return t.getRotation();
}


void drwMPolygon(Terrapin t, MPolygon mp){
     int i=0;
     stroke(255,100,100);
     t.setPenColor(200, 150, 100);
     int w; strokeWeight(w=7);
     t.x = int(mp.getCoords()[i][0]);
     t.y = int(mp.getCoords()[i][1]);
     for(i=1; i<mp.count(); i++){
        float tox = (mp.getCoords()[i][0]);
        float toy = (mp.getCoords()[i][1]);
        PPoint here = getPosition(t);
        float dis = PVector.dist(here,new PPoint(tox,toy));
        w = 4+int(random(5));
        strokeWeight(w);
        t.moveToward(int(tox),int(toy),dis);
     }
     int tox = int(mp.getCoords()[0][0]);
     int toy = int(mp.getCoords()[0][1]);
     PPoint here = getPosition(t);
     float dis = PVector.dist(here,new PPoint(tox,toy));
     t.moveToward(int(tox),int(toy),dis);
}



class VTESTER {
      PApplet applet;
      Voronoi myVoronoi; 
      float[][] myDrops;
      int MAXNRDROPS=1000;
      int nrDrops;
      Terrapin t;
      PPoint A,B,C,D,T1,T2;
      float[][] myPoints;
      VTESTER(Terrapin t, PApplet applet){
         this.t = t;
         this.applet = applet;
         myDrops = new float[MAXNRDROPS][2];
         int nrDrops = 0;
      }
void drop(Terrapin t){
     //add a point for the voronoi mesh at the terrapin's position
     if (nrDrops<MAXNRDROPS){
        strokeWeight(3);
        stroke(20,0,0);
        fill(255,0,0);
        ellipse(t.x,t.y,6,6); 
        myDrops[nrDrops][0] = t.x;
        myDrops[nrDrops][1] = t.y;
        nrDrops++;
        println("drop");
     }  else println("too many drops");
} 
void makesomething(PPoint inipos,int inidir) {
        //work with the terrapin first
        setPosition(t,inipos);
        setRotation(t,inidir);
        //create AB arbitrarily
        int s = 1;     //scale, was 20
        A2B(t,this,s);
        
        //work with the voronoi
        print("nrDrops is "); println(nrDrops);
        myPoints = new float[nrDrops][2];
        for (int i=0; i<nrDrops; i++) {
             myPoints[i][0] = myDrops[i][0];
             myPoints[i][1] = myDrops[i][1];
        }

        myVoronoi = new Voronoi( myPoints );
        
        //display the results of the voronoi
        MPolygon[] myRegions = myVoronoi.getRegions();
        for(int i=0; i<myRegions.length; i++){
        	// an array of points
	        float[][] regionCoordinates = myRegions[i].getCoords();

         	//myRegions[i].draw(this.applet); 
                drwMPolygon(t,myRegions[i]);
                // draw this shape
        }      

}
}

void draw() {
     fill(0);
     stroke(0);
     background(100,40,40);
     pdf.textMode(SHAPE);
     Terrapin t = new Terrapin(this);
     font16 = createFont("Comic Sans MS",16); 
     font32 = createFont("Comic Sans MS",16); 
     drwIntro();
     VTESTER vtester = new VTESTER(t,this);
     vtester.makesomething(new PPoint(250,350),0);
     fill(255,255,255);
          drwIntro();
     endRecord();
}

void A2B(Terrapin t, VTESTER v, int scalefactor){
     //a kind of round-cornered growing oscillation line, very arbitrary;
     //make sure not to use any absolute coordinates
     int s=scalefactor;

    for (int i=1; i<400; i++){
        strokeWeight(1);
        t.setPenColor(25, 0, 0);
        t.forward(s++);
        t.right(33+int(random(5)));
        v.drop(t);
        
    }
   
     //for (int i=2; i<80; i+=2){
     //    t.forward(s);
     //    t.left(90); v.drop(t);
     //    t.forward(i*s);
     //    t.right(90); v.drop(t);
     //    t.forward(s);
     //    t.right(90); v.drop(t);
     //    t.forward((i+1)*s);
     //    t.left(90); v.drop(t);
     //}
}

void A2BKOCH(Terrapin t, VTESTER v, int scalefactor){
     //a kind of round-cornered growing oscillation line, very arbitrary;
     //make sure not to use any absolute coordinates
     t.setPenColor(0, 0, 0);
     int s=scalefactor;
     koch(t,v,900*s,4);
}

void koch(Terrapin t, VTESTER v, float size, int n){
     if (n > 0){
	koch(t,v, size/3, n-1);
	t.left(60);
        v.drop(t);
	koch(t,v, size/3, n-1);
	t.right(120);
        v.drop(t);
        koch(t,v, size/3, n-1);
	t.left(60);
	v.drop(t);
        koch(t,v, size/3, n-1);
     }
     else t.forward(int(size)); 
}



void drwIntro(){
     textFont(font32,32);
     text("SKIN PRINT",200,50);
     textFont(font16,16);
     text("These  closed  polygons, together with  the  open polygons generated by centers on the bounding polygon, partition  the  entire plane into  n polygons. These space-filling polygons are called Thiessen polygons and the entire configuration is called a Voronoi diagram"
         , 200, 100,700,200); 
     text("The Turtle Runs Randomly to Drop Points"
         , 650, 200,700,100); 
     text("Afterwards, a Turtle draws the Polygons"
         , 650, 220,700,100); 
}

void drwA(PVector p, int scalefactor){
     //to mark the points and write their name;
     //(name writing only only for the big versions);
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(255,0,0);
     fill(255,0,0);
     ellipse(p.x,p.y,d+2,d+2); 
     if (d>5) { textFont(font32,32); text("A",p.x-10,p.y+35);}
}


void stop(){
     endRecord();
}

